/**
 * Модуль для работы с объектами КриптоПро ЭЦП Browser plug-in.
 */
CPro = {
    version: '0.0.1'
};

window.CPro = CPro;

function apply(o, c){
    if(c!==undefined){
        for(var p in c){o[p] = c[p];}
    }
    return o;
}

/**
 *  Константы плагина.
 */
CPro.const =  {
    CADESCOM_CADES_BES: 1,
    CADESCOM_CADES_X_LONG_TYPE_1: 0x5d,
    CADESCOM_BASE64_TO_BINARY: 1,

    CAPICOM_CURRENT_USER_STORE: 2,
    CAPICOM_STORE_OPEN_MAXIMUM_ALLOWED: 2,
    CAPICOM_CERTIFICATE_FIND_SUBJECT_NAME: 1,
    CAPICOM_CERTIFICATE_FIND_SHA1_HASH: 0,
    CAPICOM_CERTIFICATE_INCLUDE_WHOLE_CHAIN: 1
};

/**
 * Конструктор ошибки создания подписи
 *
 * @param message
 * @constructor
 */
CPro.SignCreateError = function(message){
    this.name = "SignCreateError";
    this.message = (message || "");
};

CPro.SignCreateError.prototype = new Error();


apply(CPro, {

    /**
     * Функция активации объектов КриптоПро ЭЦП Browser plug-in.
     *
     * @param {string} name Название объекта
     * @returns {*} Объект плагина соответствующий аргументу name.
     */
    createObject: function (name){

        function isIE() {
            return (("Microsoft Internet Explorer" == navigator.appName) || // IE < 11
                navigator.userAgent.match(/Trident\/./i)); // IE 11
        }

        function isIOS() {
            return (navigator.userAgent.match(/ipod/i) ||
              navigator.userAgent.match(/ipad/i) ||
              navigator.userAgent.match(/iphone/i));
        }

        if (isIOS()) {
            // TODO: Add IOS support
            throw new Error('IOS currently is not supported.')
        }
        if (!isIE()) {
            // В Firefox, Opera, Chrome, Safari создаются объекты NPAPI
            var pluginObject = document.getElementById("cadesplugin");
            if(!pluginObject){
                pluginObject = document.createElement("object");
                pluginObject.setAttribute('id', 'cadesplugin');
                pluginObject.setAttribute('type', 'application/x-cades');
                pluginObject.style.visibility = 'hidden';
                pluginObject.style.width = "0px";
                pluginObject.style.height = "0px";
                pluginObject.style.position = 'absolute';
                document.body.appendChild(pluginObject);
            }
            return pluginObject.CreateObject(name);
        }
        // В Internet Explorer создаются COM-объекты
        if (name.match(/X509Enrollment/i)) {
            try {
                // Объекты CertEnroll создаются через CX509EnrollmentWebClassFactory
                var objCertEnrollClassFactory = document.getElementById("certEnrollClassFactory");
                return objCertEnrollClassFactory.CreateObject(name);
            }
            catch (e) {
                throw("Для создания обьектов X509Enrollment следует настроить веб-узел на использование проверки подлинности по протоколу HTTPS");
            }
        }
        // Объекты CAPICOM и CAdESCOM создаются обычным способом
        return new ActiveXObject(name);
    },

     /**
     * Получение версии криптопровайдера.
     *
     * @returns {string} версия криптопровайдера.
     */
    getCSPVersion: function (providerName, providerType){
        var version;
        try{
            var oAbout = this.createObject("CAdESCOM.About");
            var oVersion = oAbout.CSPVersion(providerName, parseInt(providerType, 10));
            version = oVersion.toString();
        }catch(err){
            // Плагин либо не установлен, либо отключен.
        }
        return version;
    },

    /**
     * Получение версии плагина. Если плагин не установлен или отключен
     * возвращается undefined.
     *
     * @returns {string} версия плагина, если плагин доступен.
     */
    getPluginVersion: function () {

        var pluginVersion; // Версия плагина
        try {
            var oAbout = this.createObject("CAdESCOM.About");
            pluginVersion = oAbout.PluginVersion;
            if(pluginVersion === undefined)
                pluginVersion = oAbout.Version;
        }catch (err) {
            // Плагин либо не установлен, либо отключен.
        }
        return pluginVersion;
    },

    /**
     * Получение списка сертификатов пользователя.
     *
     * @returns {Array}
     */
    getCertsList: function(validOnly, actualOnDate, withPrivateKey) {

        var certificates = [];
        var oStore = this.createObject("CAPICOM.store");
        oStore.Open();
        try {
            var certCnt = oStore.Certificates.Count;
            for (var i = 1; i <= certCnt; i++) {
                var cert = oStore.Certificates.Item(i);
                if(
                    (!withPrivateKey || cert.HasPrivateKey()) &&
                    (!actualOnDate || (cert.ValidFromDate <= actualOnDate && actualOnDate <= cert.ValidToDate)) &&
                    (!validOnly  || cert.IsValid().Result)
                ){
                    certificates.push(cert);
                }
            }
        }finally{
            oStore.Close();
        }
        return certificates;
    },

    /**
     * Получение сертификата по значению хеша sha1
     *
     * @param {string} thumbprint  SHA1 хеш сертификата
     * @returns {Object} Объект сертификата
     */
    getCertByThumbprint: function(thumbprint){

        var oStore = this.createObject("CAPICOM.store");
        oStore.Open();
        var oCerts = oStore.Certificates.Find(
            CPro.const.CAPICOM_CERTIFICATE_FIND_SHA1_HASH, thumbprint);
        if (oCerts.Count) {
            return oCerts.Item(1);
        }
        oStore.Close();
    },

    /**
     * Создание подписи
     *
     * @param certificate сертификат подписи
     * @param {string} dataToSign Данные для подписания
     * @param {object} options Дополнительные опции подписи.
     * @returns {string} Подписанное сообщение
     */
    createSignature: function (certificate, dataToSign, options) {

        if(typeof  certificate == 'string'){
            certificate = this.getCertByThumbprint(certificate);
        }
        options = apply({
            signerOptions: CPro.const.CAPICOM_CERTIFICATE_INCLUDE_WHOLE_CHAIN,
            signatureType: CPro.const.CADESCOM_CADES_BES,
            detached: false,
            TSAAddress: this.TSAAddress,
        }, options);

        var oSigner = this.createObject("CAdESCOM.CPSigner");
        oSigner.Certificate = certificate;
        oSigner.TSAAddress = options.TSAAddress;
        oSigner.Options = options.signerOptions;

        var oSignedData = this.createObject("CAdESCOM.CadesSignedData");
        oSignedData.Content = dataToSign;

        try {
            var sSignedMessage = oSignedData.SignCades(
                    oSigner, options.signatureType, options.detached);
        } catch (err) {
            throw new CPro.SignCreateError(err.message);
        }
        return sSignedMessage;
    },

    /**
     * Адрес службы штампов времени.
     */
    TSAAddress: "http://testca.cryptopro.ru/tsp/tsp.srf"
});



(function(){

    function extract(from, what){
        certName = "";
        var begin = from.indexOf(what);
        if(begin>=0){
            var end = from.indexOf(', ', begin);
            certName = (end<0) ? from.substr(begin) : from.substr(begin, end - begin);
        }
        return certName;
    }

    function dateTimePutTogether(certDate){
        function check(digit){
            return (digit<10) ? "0"+digit : digit;
        }
        return check(certDate.getUTCDate())+"."+check(certDate.getUTCMonth()+1)+"." + certDate.getUTCFullYear() + " " +
            check(certDate.getUTCHours()) + ":" + check(certDate.getUTCMinutes()) + ":" + check(certDate.getUTCSeconds());
    }

    CPro.Cert = function(certObj){
        this.cert = certObj;
        this.certFromDate = new Date(this.cert.ValidFromDate);
        this.certTillDate = new Date(this.cert.ValidToDate);
    };

    apply(CPro.Cert.prototype, {
        getCertString: function(){
            return extract(this.cert.SubjectName,'CN=') + "; Выдан: " + this.getCertFromDate();
        },
        getCertFromDate: function(){
            return dateTimePutTogether(this.certFromDate);
        },
        getCertTillDate: function(){
            return dateTimePutTogether(this.certTillDate);
        },
        getPubKeyAlgorithm: function(){
            return this.cert.PublicKey().Algorithm.FriendlyName;
        },
        getSubjectName: function(){
            return extract(this.cert.SubjectName, 'CN=');
        },
        getIssuerName: function(){
            return extract(this.cert.IssuerName, 'CN=');
        }
    });

})();
